######################################################################
#:CQCC::IOPtkMenu
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2008 All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# Copyright 2001 Rational Software Corporation. All Rights Reserved
# This software contains proprietary and confidential information of
# Rational and its suppliers.  Use, disclosure or reproduction is
# prohibited without the prior express written consent of Rational.

=head1 NAME 

CQCC::IOPtkMenu - IOPtk widget class for managing menu widgets

=head1 CLASS DESCRIPTION

This class provides wrapper objects for a number of commonly used 
menu types hiding the implementation details and handling message
translation and callback setup. 

As a subclass of IOPtkWidget, it inherits some basic attribute 
management and initialization functionality.

How the IOPtkMenu object looks and behaves depends on its "subtype"
which provides a lightweight subclassing feature. Each subtype has an 
initialization function that uses Perl/TK calls to define the widget;
the subtype may have additional supporting methods to implement its
behavior. 

The currently supported subtypes include IOCommandMenu. See the method
documentation for more information.

The IOPtkPanel ClassTest method provides an extensive test of all 
of the IOPtkMenu classes. 

=head1 METHODS

=cut

######################################################################
#::Package declarations
######################################################################

require 5.006; 

package CQCC::IOPtkMenu;
use Exporter;
use CQCC::IOPtkWidget;
@ISA = qw( CQCC::IOPtkWidget );

use Tk 800.022;
             
use strict;

use CQCC::MsgCat;
use CQCC::IOBase;
use CQCC::IOPerlTK;

sub Version { return '2.1' };

######################################################################
#::Class Globals 
######################################################################

######################################################################
#::method new

=over 6

=item C<new CQCC::IOPtkMenu()>

This method builds on the IOPtkWidget constructor and performs
any additional menu specific initialization. 

=back

=cut

######################################################################

sub new { 
   my $proto = shift;
   my $class = ref($proto) || $proto;  # Object or class name

   # Attributes
   my $self = new CQCC::IOPtkWidget(@_);

   $self->{CLASSNAME} = "CQCC::IOPtkMenu";

   bless($self, $class);

   # Initialize subclass 
   $self->NewSubType();

   return $self;
}

######################################################################
#::method IOCommandMenu

=over 6

=item C<IOCommandMenu subtype>

Calling sequence using an instance of IOPerlTK called $IO:

    $IO->Menu(name, parent, "IOCommandMenu", key-values)>

This subtype method is called during initialization to define a
generic Command Menu widget. It uses the Perl/TK Frame object then
adds the menu items as buttons after translating their contents, and
sets up callbacks as appropriate.

The CALLBACK_OBJECT attribute defines which object will receive the 
callback when it occurs. If not set the MENULIST callback is 
evaluated directly. 

    CALLBACK_OBJECT => $self,

The MENULIST attribute is a list reference of list references which
contains the menu items and their callback information. This is a
sample:

    MENULIST =>
        [ ["GUI_Commit"] => "Action_Commit",
          ["GUI_Help" ]  => [ "Action_Help", "GUI_HelpMain" ],
        ],

The MENULIST is a list of pairs defining the button display message 
tag (i.e. ["GUI_Commit"]) and the callback which can be a string
to define the name of the method or a list reference defining the 
method and its arguments. The callback is managed by the IOPtkWidget
method, DoCallback. 

The WIDTH attribute defines the minimum button width in terms of
characters. 

SUPPORT POLICY: See TriggerCQCC.pm for the Rational Support Policy. 

=back

=cut

######################################################################

sub IOCommandMenu {
    my ($self) = @_;
    my @menuList = @{ $self->{MENULIST} };
    my %buttonHash;
    my $maxWidth = $self->{WIDTH} ? $self->{WIDTH} : 14;
    my $msgCat = &IOstd()->{MSGCAT};
   
    ######################################### Perl/TK code
    my @packArgs = $self->PackArgs(-fill => "both", -expand => 1 );
    my @labelArgs = $self->{TITLEMSG} ? 
                    ("-label" => $self->{TITLEMSG}) : ();
    $self->{WIDGET} = $self->{PARENT_WIDGET}->
        Frame(@labelArgs, $self->TKoptions()
             )->pack(@packArgs);

    ########################################

    # Determine maximum button width needed.
    while (@menuList) {
        my $buttonName = shift @menuList;
        my $buttonCmd = shift @menuList;
        my $buttonTitle = $msgCat->Message($buttonName);
        if (length($buttonTitle) > $maxWidth) {
            $maxWidth = length($buttonTitle);
        }
    }
    $maxWidth += 2;

    ######################################### Perl/TK code
    @menuList = @{ $self->{MENULIST} };
    while (@menuList) {
        my $buttonName = shift @menuList;
        my $buttonCmd = shift @menuList;
        my $buttonTitle = $msgCat->Message($buttonName);

        my $widget = $self->{WIDGET}->
            Button( -text => $buttonTitle,
                    -command => [ \&CQCC::IOPtkWidget::DoCallback, 
                                 $self, $buttonName, $buttonCmd  ],
                    -width => $maxWidth,
                   )->pack;
        
        $buttonHash{$buttonTitle} = $widget;
    }

    ########################################

    $self->{BUTTONHASH} = \%buttonHash;
    return $self;
}

######################################################################
#::method ManageButtons

=over 6

=item C<ManageButtons(operation, buttonName, value)>

This method can be called for any IOPtkMenu widget to modify the state
of its buttons. Operations currently supported include the following:
STATE which changes the buttonName's state to value (normal, disabled,
or active).

=back

=cut

######################################################################

sub ManageButtons {
    my ($self, $operation, $buttonName, $value) = @_;
    my $msgCat = &IOstd()->{MSGCAT};
    my $buttonTitle = $msgCat->Message($buttonName);
    my %buttonHash = %{ $self->{BUTTONHASH} };

    if ($operation eq "STATE") { 
        $buttonHash{$buttonTitle}->configure(-state => $value);
    } 
    return;
}

######################################################################
#::Tail
######################################################################

1;
